#####################################
# Render Queue for Poser Octane Plugin
#
# This code is distributed under the GNU General Public Licence (GNU GPL)
#
# Author: GGBB - Otoy forum
# v0.4 - July 2018
# Compatible with octane plugin for poser 3.08.2.88
#####################################

import pickle
import wx
import wx.lib.newevent
import time
import poser
import os
import sys

import gc

DEBUG = False

DIALOG_WIDTH = 600
DIALOG_HEIGHT = 800
BG_COLOR = wx.Colour(74, 74, 74)
FG_COLOR = wx.Colour(250, 250, 250)
TEXT_COLOR = wx.Colour(245, 245, 245)
BOX_PADDING = 10
BOX_OFFSET = 5

MyEvent, EVT_MY_EVENT = wx.lib.newevent.NewEvent()

octanePlugin = __import__("OctaneRender for Poser")
octanePlugin.Configuration.m_SuppressAnimationEndDialog = True

class Console(wx.TextCtrl):
	def __init__(self, parent, **kwargs):
		wx.TextCtrl.__init__(self, parent, style=wx.TE_MULTILINE  | wx.TE_RICH2 | wx.TE_READONLY | wx.NO_BORDER, *kwargs)
        
	def write(self, message):        
		self.SetDefaultStyle(wx.TextAttr(TEXT_COLOR, BG_COLOR))
		self.AppendText(message)

class RenderMultipleFilesForm(wx.Dialog):
	def __init__(self):
		
		man = poser.WxAuiManager()
		self.root = man.GetManagedWindow()
		wx.Dialog.__init__(self, self.root, -1, 'Render Queue for Poser Octane Plugin', size=wx.Size(DIALOG_WIDTH, DIALOG_HEIGHT), pos=wx.Point(100, 100), style=wx.DEFAULT_DIALOG_STYLE | wx.MINIMIZE_BOX)
		
		self.myRender = None
		width = self.GetClientSize()[0]
		height = self.GetClientSize()[1]
		firstColumn = BOX_OFFSET + BOX_PADDING
		secondColumn = BOX_OFFSET + BOX_PADDING + 160
		boxTitleOffset = 20
		
		self.Bind(wx.EVT_CLOSE, self.EventClose)
		self.SetBackgroundColour(BG_COLOR)
		
		self.fileListBox = wx.StaticBox(self, pos=wx.Point(BOX_OFFSET, BOX_OFFSET), size=wx.Size(width - 2*BOX_OFFSET, 240))
		t = wx.StaticText(self, label='  File Manager  ', pos=(boxTitleOffset, self.fileListBox.GetPosition()[1]))
		t.SetForegroundColour(TEXT_COLOR)
		
		self.addButton = wx.Button(self, id=-1, label='add file(s)', size=wx.Size(120,28), pos=wx.Point(BOX_OFFSET + BOX_PADDING , BOX_OFFSET + BOX_PADDING+10))
		tt = wx.ToolTip('Add file(s) in the render queue')
		tt.SetAutoPop(20000)
		self.addButton.SetToolTip(tt)
		self.Bind(wx.EVT_BUTTON, self.EventAddFile, self.addButton)
		
		self.removeButton = wx.Button(self, id=-1, label='remove file(s)', size=wx.Size(120,28), pos=wx.Point(BOX_OFFSET + BOX_PADDING + 120 + 10 , BOX_OFFSET + BOX_PADDING+10))
		tt = wx.ToolTip('Remove file(s) from the render queue')
		tt.SetAutoPop(20000)
		self.removeButton.SetToolTip(tt)
		self.Bind(wx.EVT_BUTTON, self.EventRemoveFile, self.removeButton)
		
		self.clearButton = wx.Button(self, id=-1, label='clear all', size=wx.Size(120,28), pos=wx.Point(BOX_OFFSET + BOX_PADDING + 2*(120 + 10) , BOX_OFFSET + BOX_PADDING+10))
		tt = wx.ToolTip('Clear render queue')
		tt.SetAutoPop(20000)
		self.clearButton.SetToolTip(tt)
		self.Bind(wx.EVT_BUTTON, self.EventClearFile, self.clearButton)
		
		self.fileList = wx.ListBox(self, size=wx.Size(width - 2*firstColumn, 170),pos=wx.Point(firstColumn, self.clearButton.GetPosition()[1] + self.clearButton.GetSize()[1] + BOX_PADDING),style=wx.LB_EXTENDED | wx.LB_HSCROLL | wx.LB_NEEDED_SB)
		self.fileList.SetBackgroundColour(FG_COLOR)
		
		
		
		self.optionBox = wx.StaticBox(self, pos=wx.Point(BOX_OFFSET, self.fileListBox.GetPosition()[1] + self.fileListBox.GetSize()[1] + BOX_OFFSET), size=wx.Size(width - 2*BOX_OFFSET, 180))
		t = wx.StaticText(self, label='  Options  ', pos=(boxTitleOffset, self.optionBox.GetPosition()[1]))
		t.SetForegroundColour(TEXT_COLOR)
		
		t = wx.StaticText(self, label='Import Octane Settings', pos=(firstColumn, self.optionBox.GetPosition()[1] + BOX_PADDING + 10 + 4))
		t.SetForegroundColour(TEXT_COLOR)
		self.renderFileTargetPicker = wx.FilePickerCtrl(self, wildcard="P files (*.p)|*.p", pos=wx.Point(secondColumn, self.optionBox.GetPosition()[1] + BOX_PADDING + 10), style = wx.FLP_DEFAULT_STYLE | wx.FLP_USE_TEXTCTRL)
		self.renderFileTargetPicker.SetBackgroundColour(BG_COLOR)
		tt= wx.ToolTip('Use an existing Octane setting file (.p) for all the renders. Leave blank to use Octane settings embedded in the Poser scene.')
		tt.SetAutoPop(20000)		
		t.SetToolTip(tt)
		
		
		t = wx.StaticText(self, label='Resolution in Pixel', pos=(firstColumn, self.renderFileTargetPicker.GetPosition()[1] + self.renderFileTargetPicker.GetSize()[1] + 5 + 4))
		t.SetForegroundColour(TEXT_COLOR)
		tt = wx.ToolTip('Set the resolution for all the renders. If an Octane setting file is provided, its resolution will be overwritten. Leave blank to use the resolution embedded in the Poser scene or in the Octane setting file.')
		tt.SetAutoPop(20000)
		t.SetToolTip(tt)
		self.widthTextCtrl = wx.TextCtrl(self, size=wx.Size(60,-1), pos=wx.Point(secondColumn, self.renderFileTargetPicker.GetPosition()[1] + self.renderFileTargetPicker.GetSize()[1] + 5),style=wx.TE_RIGHT)
		t = wx.StaticText(self, label='  x  ', pos=wx.Point(self.widthTextCtrl.GetPosition()[0] + self.widthTextCtrl.GetSize()[0], self.widthTextCtrl.GetPosition()[1] + 4))
		t.SetForegroundColour(TEXT_COLOR)
		self.heightTextCtrl = wx.TextCtrl(self, size=wx.Size(60,-1), pos=wx.Point(t.GetPosition()[0] + t.GetSize()[0], self.widthTextCtrl.GetPosition()[1]),style=wx.TE_RIGHT)
		
		t = wx.StaticText(self, label='Max. Samples', pos=wx.Point(firstColumn, self.widthTextCtrl.GetPosition()[1] + self.widthTextCtrl.GetSize()[1] + 4 + 5))
		t.SetForegroundColour(TEXT_COLOR)
		self.maxsamplesTextCtrl = wx.TextCtrl(self, size=wx.Size(60,-1), pos=wx.Point(secondColumn, self.widthTextCtrl.GetPosition()[1] + self.widthTextCtrl.GetSize()[1] + 5),style=wx.TE_RIGHT)
		
		tt = wx.ToolTip('Set the max samples per pixel value for all renders. If an Octane setting file is provided, its Max. Samples value will be overwritten. Leave blank to use the Max. Samples value embedded in the Poser scene or in the Octane setting file.')
		tt.SetAutoPop(20000)
		t.SetToolTip(tt)
		
		t = wx.StaticText(self, label='Keep Camera Settings', pos=wx.Point(firstColumn, self.maxsamplesTextCtrl.GetPosition()[1] + self.maxsamplesTextCtrl.GetSize()[1] + 4 + 5))
		t.SetForegroundColour(TEXT_COLOR)
		self.keepcameraCheckBox = wx.CheckBox(self, pos=wx.Point(secondColumn, self.maxsamplesTextCtrl.GetPosition()[1] + self.maxsamplesTextCtrl.GetSize()[1] + 5 + 4),style=wx.TE_RIGHT)
		self.keepcameraCheckBox.SetValue(True)
		
		tt = wx.ToolTip('Keep camera settings embedded in the Poser scene. It is usefull if you want to import an Octane setting file but keep camera settings embedded in the Poser scene.')
		tt.SetAutoPop(20000)
		t.SetToolTip(tt)
		
		t = wx.StaticText(self, label='Render Animation', pos=wx.Point(firstColumn, self.keepcameraCheckBox.GetPosition()[1] + self.keepcameraCheckBox.GetSize()[1] + 12))
		t.SetForegroundColour(TEXT_COLOR)
		self.renderAnimation = wx.CheckBox(self, pos=wx.Point(secondColumn, self.keepcameraCheckBox.GetPosition()[1] + self.keepcameraCheckBox.GetSize()[1] + 12),style=wx.TE_RIGHT)
		self.renderAnimation.SetValue(False)
		
		tt = wx.ToolTip('Check to render animation. Set the animation parameters using the Octane plugin animation tab.')
		tt.SetAutoPop(20000)
		t.SetToolTip(tt)
		
		
		self.infoBox = wx.StaticBox(self, pos=wx.Point(BOX_OFFSET, BOX_OFFSET + self.optionBox.GetPosition()[1] + self.optionBox.GetSize()[1] + 5), size=wx.Size(DIALOG_WIDTH - 25, 220))
		t = wx.StaticText(self, label='  Info  ', pos=(20, self.infoBox.GetPosition()[1]))
		t.SetForegroundColour(TEXT_COLOR)
		
		myConsole = Console(self)
		sys.stdout = myConsole
		myConsole.SetSize(wx.Size(width - 2*firstColumn, 195))
		myConsole.SetPosition(wx.Point(firstColumn,self.infoBox.GetPosition()[1] + BOX_OFFSET + 15))
		myConsole.SetBackgroundColour(BG_COLOR)
		
		print "welcome to the Poser OctanePlugin rendering batch"
		print "this a test version - use at your own risk"
		
		self.startButton = wx.Button(self, id=-1, label='start rendering', size=wx.Size(120,28), pos=wx.Point(DIALOG_WIDTH/2 - 120 - 5, self.GetClientSize()[1] - 28 - 10))
		self.Bind(wx.EVT_BUTTON, self.EventStart, self.startButton)
		
		self.abortButton = wx.Button(self, id=-1, label='abort rendering', size=wx.Size(120,28), pos=wx.Point(DIALOG_WIDTH/2 + 5, self.GetClientSize()[1] - 28 - 10))
		self.Bind(wx.EVT_BUTTON, self.EventAbort, self.abortButton)
		
		self.root.Bind(EVT_MY_EVENT, self.EventFinished)
		self.WaitMode()
		
	def EventFinished(self,evt):
		self.WaitMode()
	
	def EventAddFile(self,event):
		openFileDialog  = wx.FileDialog(self, message="Select pz3 file(s)", wildcard="PZ3 files (*.pz3)|*.pz3", style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST | wx.FD_MULTIPLE | wx.FD_CHANGE_DIR)
		
		if openFileDialog.ShowModal() == wx.ID_CANCEL:
			return
		filenames = openFileDialog.GetPaths()
		self.fileList.InsertItems(filenames,0)
	
	def EventRemoveFile(self,event):
		sel = self.fileList.GetSelections()
		sel = sorted(sel,reverse=True)
		for i in sel:
			self.fileList.Delete(i)
		
	def EventClearFile(self,event):
		self.fileList.Clear()
	
	def EventClose(self, event):
		dlg = wx.MessageDialog(None,'Are you sure you want to close the render queue?','Warning',wx.YES_NO + wx.NO_DEFAULT)
		
		if dlg.ShowModal() != wx.ID_YES:
			return
		
		self.Abort()
		
		if octanePlugin.UiManager.IsViewportOpen():
			octanePlugin.UiManager.GetViewportForm().CloseWindow()
			
		if octanePlugin.UiManager.IsSetupWindowOpen():
			octanePlugin.UiManager.GetSetupForm().CloseWindow()
		
		self.root.Unbind(EVT_MY_EVENT)
		sys.stdout = sys.__stdout__
		self.Destroy()

	def EventStart(self, event):
		# dlg = wx.MessageDialog(None,'Are you sure you want to start the batch rendering?','Warning',wx.YES_NO + wx.YES_DEFAULT)
		# if dlg.ShowModal() != wx.ID_YES:
			# return
		self.StartRendering()
			
	def EventAbort(self,event):
		dlg = wx.MessageDialog(None,'Are you sure you want to abort the batch rendering?','Warning',wx.YES_NO + wx.NO_DEFAULT)
		if dlg.ShowModal() != wx.ID_YES:
			return
			
		self.Abort()
		
	def StartRendering(self):
		files = self.fileList.GetItems()
		
		# get resolution
		width = self.widthTextCtrl.GetLineText(0)
		height = self.heightTextCtrl.GetLineText(0)
		if (width == "") | (height == ""):
			resolution = None
		else:
			resolution = [int(float(width)),int(float(height))]
		
		# get rendertarget file
		renderTargetFile = self.renderFileTargetPicker.GetPath()
		if renderTargetFile == "":
			renderTargetFile = None
		
		# get maxsamples
		maxsamples = self.maxsamplesTextCtrl.GetLineText(0)
		if maxsamples == "":
			maxsamples = None
		else:
			maxsamples = int(float(maxsamples))
		
		# get keepCam
		keepCamera = self.keepcameraCheckBox.GetValue()
		
		# is animation
		isAnimation = self.renderAnimation.GetValue()
		
		if DEBUG:
			print "files:", files
			print "renderTargetFile:", renderTargetFile
			print "resolution:", resolution
			print "maxsamples:", maxsamples
			print "keepCamera:", keepCamera
			print "isAnimation:", isAnimation
		
		self.RenderMode()
		self.myRender = RenderMultipleFiles(files,renderTargetFile,maxsamples,resolution,keepCamera,isAnimation)
		self.myRender.run()
		
	def Abort(self):
		if self.myRender != None:
			self.myRender.abort()
		self.WaitMode()
	
	def WaitMode(self):
		self.startButton.Enable()
		self.addButton.Enable()
		self.removeButton.Enable()
		self.clearButton.Enable()
		self.renderFileTargetPicker.Enable()
		self.abortButton.Disable()
		return
	
	def RenderMode(self):
		self.startButton.Disable()
		self.addButton.Disable()
		self.removeButton.Disable()
		self.clearButton.Disable()
		self.renderFileTargetPicker.Disable()
		self.abortButton.Enable()
		return

class RenderMultipleFiles():
	def __init__(self,fileList, renderTargetFile=None, maxSamples=None, resolution=None, keepCamera=True, isAnimation=False):
		man = poser.WxAuiManager()
		self.root = man.GetManagedWindow()
		self.renderTargetFile = renderTargetFile
		self.maxSamples = maxSamples
		self.resolution = resolution
		self.keepCamera = keepCamera
		self.fileList = fileList
		self.isAnimation = isAnimation
		self.nFiles = len(self.fileList)
		self.fileIdx = 0
		self.isRenderFinished = False
		self.timer = None
		self.currentFile = None
		self.terminated = False
	
	def run(self):
		print "queue rendering started at",time.strftime("%H:%M:%S %d-%m-%Y")
		print "interacting with Octane plugin setup or viewport may crash Poser"
		if (self.renderTargetFile is not None):
			if os.path.isfile(self.renderTargetFile):
				if os.path.splitext(self.renderTargetFile)[1] != ".p":
					print "warning: rendertargetfile", self.renderTargetFile, "is not valid. Discard rendertarget file."
					self.renderTargetFile = None
			else:
				print "warning: rendertargetfile", self.renderTargetFile, "does not exist. Discard rendertarget file."
				self.renderTargetFile = None
		self.startTimer()
		self.renderFile()
		return
		
	def on_timer(self, event):
		stats = octanePlugin.OctaneWrapper.ResultStatistics()
		octanePlugin.OctaneWrapper.GetResultStatistics(stats)
		
		if self.isAnimation:
			maxFrame = int(octanePlugin.UiManager.GetSetupForm().endFrameTextCtrl.GetValue())
			
			if (not octanePlugin.UiManager.GetSetupForm().renderingAnimation) & (stats.samplesPerPixel > 0):
				self.isRenderFinished = False
				self.renderFile()
			else:
				pass

		else:		
			if (stats.samplesPerPixel > 0) & (stats.samplesPerPixel >= stats.maxSamplesPerPixel):
				self.isRenderFinished = True
			if self.isRenderFinished == True:
				self.isRenderFinished = False
				self.saveRender()
				self.renderFile()
	
	def saveRender(self):
		splitExt = os.path.splitext(self.currentFile)
		fileroot = splitExt[0]
		file = fileroot + '.png'
		file = file.encode('utf-8')
		octanePlugin.OctaneWrapper.SaveImage(file , 1 , 0)
		print "render saved as", file
	
	def abort(self):
		self.stopTimer()
		if octanePlugin.UiManager.IsViewportOpen():
			octanePlugin.UiManager.GetViewportForm().CloseWindow()
				
		if octanePlugin.UiManager.IsSetupWindowOpen():
			octanePlugin.UiManager.GetSetupForm().CloseWindow()
			
		print "render queue aborted by user at", time.strftime("%H:%M:%S %d-%m-%Y")
		
	def startTimer(self):
		TIMER_ID = 105
		self.timer = wx.Timer(self.root)
		self.root.Bind(wx.EVT_TIMER, self.on_timer, self.timer)
		self.timer.Start(1000)

	def stopTimer(self):
		if self.timer is not None:
			self.timer.Stop()
		self.timer = None
		return
	
	def renderFile(self):
		if self.fileIdx < self.nFiles:
			self.currentFile = self.fileList[self.fileIdx]
			self.fileIdx = self.fileIdx + 1
			if os.path.isfile(self.currentFile) & (os.path.splitext(self.currentFile)[1] == '.pz3'):
				print "rendering file", self.currentFile,"- please wait..."
				if octanePlugin.UiManager.IsViewportOpen():
					octanePlugin.UiManager.GetViewportForm().CloseWindow()
					# for win in wx.GetTopLevelWindows():
						# if isinstance(win, wx.MessageDialog):
							# if win.IsModal():
								# print "destroying modal window"
								# win.Destroy()
					
				if octanePlugin.UiManager.IsSetupWindowOpen():
					octanePlugin.UiManager.GetSetupForm().CloseWindow()

				poser.CloseDocument(1)
				poser.OpenDocument(self.currentFile)
				
				if self.keepCamera:
					camera = octanePlugin.Globals.m_SavedTreeInfo['rendertarget']['camera']
					
				if self.renderTargetFile is not None:
					f = open(self.renderTargetFile, 'rb')
					octanePlugin.Globals.m_SavedTreeInfo = pickle.load(f)
					f.close()
					if self.keepCamera:
						octanePlugin.Globals.m_SavedTreeInfo['rendertarget']['camera'] = camera
				elif octanePlugin.Globals.m_SavedTreeInfo == None:
					# TODO: skip this file
					pass
					
					
				
				if self.maxSamples is not None:
					octanePlugin.Globals.m_SavedTreeInfo['rendertarget']['kernel']['maxsamples']['value'] = self.maxSamples
					
				if self.resolution is not None:
					octanePlugin.Globals.m_SavedTreeInfo['rendertarget']['filmSettings']['resolution']['value'] = self.resolution
				
				if self.isAnimation:
					outputDir = os.path.splitext(self.currentFile)
					if not os.path.isdir(outputDir[0]):
						os.mkdir(outputDir[0])
					octanePlugin.Globals.m_SavedTreeInfo['outputFolder'] = outputDir[0]
					octanePlugin.UiManager.ShowSetupForm()
					#octanePlugin.UiManager.RefreshSetupForm()
					octanePlugin.Globals.m_SetupForm.EventRenderAnimation(None)
					
				else:
					octanePlugin.UiManager.ShowSetupForm()
					#octanePlugin.UiManager.RefreshSetupForm()
					octanePlugin.UiManager.ShowViewportForm()
					octanePlugin.Node.ClearNodeEditControls(octanePlugin.UiManager.GetSetupForm().p3.optionsTreePanel.nodePanel)
					
			else:
				# self.abort()
				print "warning: file", self.currentFile, "does not exist or is not a valid Poser file. Skip file."
				self.renderFile()
				return
		else:
			self.stopTimer()
			evt = MyEvent()
			wx.PostEvent(self.root,evt)
			print "queue rendering finished at ",time.strftime("%H:%M:%S %d-%m-%Y")
			
		return

myClass = RenderMultipleFilesForm()
myClass.Show()
